@@ -32,7 +32,7 @@ from pytensor.graph.basic import Apply, Variable
 from pytensor.graph.rewriting.basic import in2out
 from pytensor.graph.utils import MetaType
 from pytensor.tensor.basic import as_tensor_variable
-from pytensor.tensor.random.op import RandomVariable
+from pytensor.tensor.random.op import RandomVariable, RNGConsumerOp
 from pytensor.tensor.random.rewriting import local_subtensor_rv_lift
 from pytensor.tensor.random.utils import normalize_size_param
 from pytensor.tensor.rewriting.shape import ShapeFeature
@@ -47,10 +47,11 @@ from pymc.distributions.shape_utils import (
     convert_shape,
     convert_size,
     find_size,
+    maybe_resize,
     rv_size_is_none,
     shape_from_dims,
 )
-from pymc.logprob.abstract import MeasurableOp, _icdf, _logcdf, _logprob
+from pymc.logprob.abstract import MeasurableOp, _icdf, _logccdf, _logcdf, _logprob
 from pymc.logprob.basic import logp
 from pymc.logprob.rewriting import logprob_rewrites_db
 from pymc.printing import str_for_dist
@@ -120,46 +121,55 @@ class DistributionMeta(ABCMeta):
                         SymbolicRandomVariable.get_input_output_type_idxs(extended_signature)
                     )
 
-            class_change_dist_size = clsdict.get("change_dist_size")
-            if class_change_dist_size:
+            def _extract_dist_params_and_size(
+                op, params
+            ) -> tuple[tuple[Variable, ...], Variable | None]:
+                size = None
+                if isinstance(op, RandomVariable):
+                    rng, size, *dist_params = params
+                else:
+                    if size_idx is not None:
+                        size = params[size_idx]
+                    if params_idxs is not None:
+                        dist_params = [params[i] for i in params_idxs]
+                    else:
+                        dist_params = params
+                return dist_params, size
+
+            if (class_change_dist_size := clsdict.get("change_dist_size")) is not None:
 
                 @_change_dist_size.register(rv_type)
                 def change_dist_size(op, rv, new_size, expand):
                     return class_change_dist_size(rv, new_size, expand)
 
-            class_logp = clsdict.get("logp")
-            if class_logp:
+            if (class_logp := clsdict.get("logp")) is not None:
 
                 @_logprob.register(rv_type)
-                def logp(op, values, *dist_params, **kwargs):
-                    if isinstance(op, RandomVariable):
-                        rng, size, *dist_params = dist_params
-                    elif params_idxs:
-                        dist_params = [dist_params[i] for i in params_idxs]
+                def logp(op, values, *params, **kwargs):
+                    dist_params, size = _extract_dist_params_and_size(op, params)
                     [value] = values
-                    return class_logp(value, *dist_params)
+                    return maybe_resize(class_logp(value, *dist_params), size)
 
-            class_logcdf = clsdict.get("logcdf")
-            if class_logcdf:
+            if (class_logcdf := clsdict.get("logcdf")) is not None:
 
                 @_logcdf.register(rv_type)
-                def logcdf(op, value, *dist_params, **kwargs):
-                    if isinstance(op, RandomVariable):
-                        rng, size, *dist_params = dist_params
-                    elif params_idxs:
-                        dist_params = [dist_params[i] for i in params_idxs]
-                    return class_logcdf(value, *dist_params)
+                def logcdf(op, value, *params, **kwargs):
+                    dist_params, size = _extract_dist_params_and_size(op, params)
+                    return maybe_resize(class_logcdf(value, *dist_params), size)
 
-            class_icdf = clsdict.get("icdf")
-            if class_icdf:
+            if (class_logccdf := clsdict.get("logccdf")) is not None:
+
+                @_logccdf.register(rv_type)
+                def logccdf(op, value, *params, **kwargs):
+                    dist_params, size = _extract_dist_params_and_size(op, params)
+                    return maybe_resize(class_logccdf(value, *dist_params), size)
+
+            if (class_icdf := clsdict.get("icdf")) is not None:
 
                 @_icdf.register(rv_type)
-                def icdf(op, value, *dist_params, **kwargs):
-                    if isinstance(op, RandomVariable):
-                        rng, size, *dist_params = dist_params
-                    elif params_idxs:
-                        dist_params = [dist_params[i] for i in params_idxs]
-                    return class_icdf(value, *dist_params)
+                def icdf(op, value, *params, **kwargs):
+                    dist_params, size = _extract_dist_params_and_size(op, params)
+                    return maybe_resize(class_icdf(value, *dist_params), size)
 
             class_moment = clsdict.get("moment")
             if class_moment:
@@ -170,21 +180,18 @@ class DistributionMeta(ABCMeta):
 
                 clsdict["support_point"] = class_moment
 
-            class_support_point = clsdict.get("support_point")
-
-            if class_support_point:
+            if (class_support_point := clsdict.get("support_point")) is not None:
 
                 @_support_point.register(rv_type)
-                def support_point(op, rv, *dist_params):
-                    if isinstance(op, RandomVariable):
-                        rng, size, *dist_params = dist_params
-                        return class_support_point(rv, size, *dist_params)
-                    elif params_idxs and size_idx is not None:
-                        size = dist_params[size_idx]
-                        dist_params = [dist_params[i] for i in params_idxs]
-                        return class_support_point(rv, size, *dist_params)
-                    else:
+                def support_point(op, rv, *params):
+                    dist_params, size = _extract_dist_params_and_size(op, params)
+                    # TODO: Move size out of support_point implementations
+                    # and get rid of this differential signature
+                    # Note that support_point is aligned with shape, not size
+                    if size is None:
                         return class_support_point(rv, *dist_params)
+                    else:
+                        return class_support_point(rv, size, *dist_params)
 
             # Register the PyTensor rv_type as a subclass of this PyMC Distribution type.
             new_cls.register(rv_type)
@@ -207,7 +214,7 @@ class _class_or_instance_property(property):
         return self.fget(owner_self if owner_self is not None else owner_cls)
 
 
-class SymbolicRandomVariable(MeasurableOp, OpFromGraph):
+class SymbolicRandomVariable(MeasurableOp, RNGConsumerOp, OpFromGraph):
     """Symbolic Random Variable.
 
     This is a subclasse of `OpFromGraph` which is used to encapsulate the symbolic
@@ -294,7 +301,10 @@ class SymbolicRandomVariable(MeasurableOp, OpFromGraph):
     @staticmethod
     def get_input_output_type_idxs(
         extended_signature: str | None,
-    ) -> tuple[tuple[tuple[int], int | None, tuple[int]], tuple[tuple[int], tuple[int]]]:
+    ) -> tuple[
+        tuple[tuple[int, ...], int | None, tuple[int, ...]],
+        tuple[tuple[int, ...], tuple[int, ...]],
+    ]:
         """Parse extended_signature and return indexes for *[rng], [size] and parameters as well as outputs."""
         if extended_signature is None:
             raise ValueError("extended_signature must be provided")
@@ -367,8 +377,29 @@ class SymbolicRandomVariable(MeasurableOp, OpFromGraph):
 
         kwargs.setdefault("inline", True)
         kwargs.setdefault("strict", True)
+        # Many RVS have a size argument, even when this is `None` and is therefore unused
+        kwargs.setdefault("on_unused_input", "ignore")
+        if hasattr(self, "name"):
+            kwargs.setdefault("name", self.name)
         super().__init__(*args, **kwargs)
 
+    def make_node(self, *inputs):
+        # If we try to build the RV with a different size type (vector -> None or None -> vector)
+        # We need to rebuild the Op with new size type in the inner graph
+        if self.extended_signature is not None:
+            (rng_arg_idxs, size_arg_idx, param_idxs), _ = self.get_input_output_type_idxs(
+                self.extended_signature
+            )
+            if size_arg_idx is not None and len(rng_arg_idxs) == 1:
+                new_size_type = normalize_size_param(inputs[size_arg_idx]).type
+                if not self.input_types[size_arg_idx].is_super(new_size_type):
+                    params = [inputs[idx] for idx in param_idxs]
+                    size = inputs[size_arg_idx]
+                    rng = inputs[rng_arg_idxs[0]]
+                    return self.rebuild_rv(*params, size=size, rng=rng).owner
+
+        return super().make_node(*inputs)
+
     def update(self, node: Apply) -> dict[Variable, Variable]:
         """Symbolic update expression for input random state variables.
 
