@@ -1,4 +1,4 @@
-#   Copyright 2024 The PyMC Developers
+#   Copyright 2024 - present The PyMC Developers
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -12,23 +12,26 @@
 #   See the License for the specific language governing permissions and
 #   limitations under the License.
 import abc
-import sys
 import warnings
 
 from abc import ABC
-from typing import TypeAlias, cast
+from typing import TypeAlias
 
 import numpy as np
 import pytensor.tensor as pt
 
+from pytensor import shared
 from pytensor.graph.replace import clone_replace
+from pytensor.link.jax import JAXLinker
+from pytensor.tensor.random.type import RandomGeneratorType
+from rich.progress import TextColumn
+from rich.table import Column
 from scipy.special import logsumexp
 from scipy.stats import multivariate_normal
 
-from pymc.backends.ndarray import NDArray
 from pymc.blocking import DictToArrayBijection
 from pymc.initial_point import make_initial_point_expression
-from pymc.model import Point, modelcontext
+from pymc.model import modelcontext
 from pymc.pytensorf import (
     compile,
     floatX,
@@ -127,6 +130,11 @@ class SMC_KERNEL(ABC):
 
     """
 
+    stats_dtypes_shapes: dict[str, tuple[type, list]] = {
+        "log_marginal_likelihood": (float, []),
+        "beta": (float, []),
+    }
+
     def __init__(
         self,
         draws=2000,
@@ -134,6 +142,7 @@ class SMC_KERNEL(ABC):
         model=None,
         random_seed=None,
         threshold=0.5,
+        compile_kwargs: dict | None = None,
     ):
         """
         Initialize the SMC_kernel class.
@@ -154,6 +163,8 @@ class SMC_KERNEL(ABC):
             Determines the change of beta from stage to stage, i.e.indirectly the number of stages,
             the higher the value of `threshold` the higher the number of stages. Defaults to 0.5.
             It should be between 0 and 1.
+        compile_kwargs: dict, optional
+            Keyword arguments passed to pytensor.function
 
         Attributes
         ----------
@@ -166,63 +177,110 @@ class SMC_KERNEL(ABC):
         if threshold < 0 or threshold > 1:
             raise ValueError(f"Threshold value {threshold} must be between 0 and 1")
         self.threshold = threshold
-        self.model = model
+
+        model = modelcontext(model)
         self.rng = np.random.default_rng(seed=random_seed)
 
-        self.model = modelcontext(model)
-        self.variables = self.model.value_vars
+        self.variables = model.value_vars
 
-        self.var_info = {}
-        self.tempered_posterior = None
-        self.prior_logp = None
-        self.likelihood_logp = None
-        self.tempered_posterior_logp = None
-        self.prior_logp_func = None
-        self.likelihood_logp_func = None
-        self.log_marginal_likelihood = 0
-        self.beta = 0
+        self.var_info: dict[str, tuple] = {}
+        self.tempered_posterior: np.ndarray
+        self.prior_logp: np.ndarray | None = None
+        self.likelihood_logp: np.ndarray | None = None
+        self.tempered_posterior_logp: np.ndarray | None = None
+        self.log_marginal_likelihood: float = 0.0
+        self.beta = 0.0
         self.iteration = 0
-        self.resampling_indexes = None
+        self.resampling_indexes: np.ndarray | None = None
         self.weights = np.ones(self.draws) / self.draws
 
+        initial_point = model.initial_point(random_seed=self.rng.integers(2**30))
+
+        for v in self.variables:
+            self.var_info[v.name] = (initial_point[v.name].shape, initial_point[v.name].size)
+
+        shared = make_shared_replacements(initial_point, self.variables, model)
+        compile_kwargs = compile_kwargs if compile_kwargs is not None else {}
+
+        # If a model has no observed variables, the likelihood_logp will have unused inputs, which can be safely
+        # ignored.
+        compile_kwargs.update({"on_unused_input": "ignore"})
+
+        self.prior_logp_func = _logp_forw(
+            initial_point, [model.varlogp], self.variables, shared, compile_kwargs
+        )
+        self.likelihood_logp_func = _logp_forw(
+            initial_point, [model.datalogp], self.variables, shared, compile_kwargs
+        )
+
+        prior_expression = make_initial_point_expression(
+            free_rvs=model.free_RVs,
+            rvs_to_transforms=model.rvs_to_transforms,
+            initval_strategies={
+                **model.rvs_to_initial_values,
+            },
+            default_strategy="prior",
+            return_transformed=True,
+        )
+
+        self._prior_expression = prior_expression
+        self._prior_var_names = [model.rvs_to_values[rv].name for rv in model.free_RVs]
+
+    def set_rng(self, rng: np.random.Generator):
+        """
+        Copy compiled functions, updating their random number generators.
+
+        This is necessary because these functions were compiled once at initialization, then pickled
+        and sent to worker processes. Each worker needs its own RNG state to ensure independent sampling,
+        so we replace the shared RNGs in the compiled functions with new ones created from the provided `rng`.
+
+        This method copies the functions, so it is expensive and should only be called once per worker!
+        """
+
+        def make_rng_swaps(fn, rng):
+            shared_rngs = [
+                var for var in fn.get_shared() if isinstance(var.type, RandomGeneratorType)
+            ]
+            n_shared_rngs = len(shared_rngs)
+            if n_shared_rngs > 0 and isinstance(fn.maker.linker, JAXLinker):
+                raise NotImplementedError(
+                    f"JAX rngs cannot be replaced after compilation. {self}.set_rng will fail to "
+                    f"properly update random seeds between chains, resulting in non-independent "
+                    f"sampling."
+                )
+
+            return {
+                old_shared_rng: shared(new_rng, borrow=True)
+                for old_shared_rng, new_rng in zip(
+                    shared_rngs, rng.spawn(n_shared_rngs), strict=True
+                )
+            }
+
+        self.rng = rng
+        self.prior_logp_func = self.prior_logp_func.copy(
+            swap=make_rng_swaps(self.prior_logp_func, self.rng)
+        )
+        self.likelihood_logp_func = self.likelihood_logp_func.copy(
+            swap=make_rng_swaps(self.likelihood_logp_func, self.rng)
+        )
+
     def initialize_population(self) -> dict[str, np.ndarray]:
         """Create an initial population from the prior distribution."""
-        sys.stdout.write(" ")  # see issue #5828
         with warnings.catch_warnings():
             warnings.filterwarnings(
                 "ignore", category=UserWarning, message="The effect of Potentials"
             )
 
-            model = self.model
+            prior_values = draw(self._prior_expression, draws=self.draws, random_seed=self.rng)
+            dict_prior = dict(zip(self._prior_var_names, prior_values))
 
-            prior_expression = make_initial_point_expression(
-                free_rvs=model.free_RVs,
-                rvs_to_transforms=model.rvs_to_transforms,
-                initval_strategies={
-                    **model.rvs_to_initial_values,
-                },
-                default_strategy="prior",
-                return_transformed=True,
-            )
-            prior_values = draw(prior_expression, draws=self.draws, random_seed=self.rng)
-
-            names = [model.rvs_to_values[rv].name for rv in model.free_RVs]
-            dict_prior = {k: np.stack(v) for k, v in zip(names, prior_values)}
-
-        return cast(dict[str, np.ndarray], dict_prior)
+        return dict_prior
 
     def _initialize_kernel(self):
-        """Create variables and logp function necessary to run SMC kernel.
-
-        This method should not be overwritten. If needed, use `setup_kernel`
-        instead.
+        """Initialize particles and compute their prior and likelihood logp.
 
+        This method should not be overwritten. If needed, use `setup_kernel` instead.
         """
-        # Create dictionary that stores original variables shape and size
-        initial_point = self.model.initial_point(random_seed=self.rng.integers(2**30))
-        for v in self.variables:
-            self.var_info[v.name] = (initial_point[v.name].shape, initial_point[v.name].size)
-        # Create particles bijection map
         if self.start:
             init_rnd = self.start
         else:
@@ -230,21 +288,12 @@ class SMC_KERNEL(ABC):
 
         population = []
         for i in range(self.draws):
-            point = Point({v.name: init_rnd[v.name][i] for v in self.variables}, model=self.model)
+            point = {v.name: init_rnd[v.name][i] for v in self.variables}
             population.append(DictToArrayBijection.map(point).data)
 
         self.tempered_posterior = np.array(floatX(population))
 
-        # Initialize prior and likelihood log probabilities
-        shared = make_shared_replacements(initial_point, self.variables, self.model)
-
-        self.prior_logp_func = _logp_forw(
-            initial_point, [self.model.varlogp], self.variables, shared
-        )
-        self.likelihood_logp_func = _logp_forw(
-            initial_point, [self.model.datalogp], self.variables, shared
-        )
-
+        # Evaluate prior and likelihood for initial particles
         priors = [self.prior_logp_func(sample) for sample in self.tempered_posterior]
         likelihoods = [self.likelihood_logp_func(sample) for sample in self.tempered_posterior]
 
@@ -311,57 +360,96 @@ class SMC_KERNEL(ABC):
         """Apply kernel-specific perturbation to the particles once per stage."""
         pass
 
+    @abc.abstractmethod
     def sample_stats(self) -> SMCStats:
         """Stats to be saved at the end of each stage.
 
         These stats will be saved under `sample_stats` in the final InferenceData object.
         """
-        return {
-            "log_marginal_likelihood": self.log_marginal_likelihood if self.beta == 1 else np.nan,
-            "beta": self.beta,
-        }
+        pass
 
+    def step(self) -> SMCStats:
+        """Perform a single SMC stage: resample, tune, and mutate."""
+        self.resample()
+        self.tune()
+        self.mutate()
+
+        return self.sample_stats()
+
+    @abc.abstractmethod
     def sample_settings(self) -> SMCSettings:
         """SMC_kernel settings to be saved once at the end of sampling.
 
         These stats will be saved under `sample_stats` in the final InferenceData object.
-
         """
-        return {
-            "_n_draws": self.draws,  # Default property name used in `SamplerReport`
-            "threshold": self.threshold,
+        pass
+
+    def _reset_state(self):
+        """Reset the sampling state for a new run."""
+        self.tempered_posterior = np.empty(0)
+        self.prior_logp = None
+        self.likelihood_logp = None
+        self.tempered_posterior_logp = None
+        self.log_marginal_likelihood = 0.0
+        self.beta = 0.0
+        self.iteration = 0
+        self.resampling_indexes = None
+        self.weights = np.ones(self.draws) / self.draws
+
+    def initialize(self, start: dict | None, rng: np.random.Generator) -> None:
+        """Initialize the kernel for sampling.
+
+        Parameters
+        ----------
+        start : dict or None
+            Starting point in parameter space, or None to sample from prior.
+        rng : np.random.Generator
+            Random number generator for this chain.
+        """
+        self.start = start
+        self.rng = rng
+        self._reset_state()
+        self.set_rng(rng)
+        self._initialize_kernel()
+        self.setup_kernel()
+
+    @staticmethod
+    def _progressbar_config(n_chains=1):
+        """Configure progress bar columns for SMC sampling.
+
+        Returns columns to display and initial stats values.
+        """
+        columns = [
+            TextColumn("{task.fields[beta]:.4f}", table_column=Column("Beta", ratio=1)),
+        ]
+
+        stats = {
+            "beta": [0.0] * n_chains,
         }
 
-    def _posterior_to_trace(self, chain=0) -> NDArray:
-        """Save results into a PyMC trace.
+        return columns, stats
 
-        This method should not be overwritten.
-        """
-        length_pos = len(self.tempered_posterior)
-        varnames = [v.name for v in self.variables]
+    @staticmethod
+    def _make_progressbar_update_functions():
+        """Create functions to update progress bar statistics."""
 
-        with self.model:
-            strace = NDArray(name=self.model.name)
-            strace.setup(length_pos, chain)
-        for i in range(length_pos):
-            value = []
-            size = 0
-            for var in self.variables:
-                shape, new_size = self.var_info[var.name]
-                var_samples = self.tempered_posterior[i][size : size + new_size]
-                # Round discrete variable samples. The rounded values were the ones
-                # actually used in the logp evaluations (see logp_forw)
-                if var.dtype in discrete_types:
-                    var_samples = np.round(var_samples).astype(var.dtype)
-                value.append(var_samples.reshape(shape))
-                size += new_size
-            strace.record(point=dict(zip(varnames, value)))
-        return strace
+        def update_stats(stats):
+            return {
+                "beta": stats.get("beta", 0.0),
+            }
+
+        return (update_stats,)
 
 
 class IMH(SMC_KERNEL):
     """Independent Metropolis-Hastings SMC_kernel."""
 
+    stats_dtypes_shapes: dict[str, tuple[type, list]] = {
+        "log_marginal_likelihood": (float, []),
+        "beta": (float, []),
+        "accept_rate": (float, []),
+    }
+
     def __init__(self, *args, correlation_threshold=0.01, **kwargs):
         """
         Create the Independent Metropolis-Hastings SMC kernel object.
@@ -435,23 +523,19 @@ class IMH(SMC_KERNEL):
         self.acc_rate = np.mean(ac_)
 
     def sample_stats(self) -> SMCStats:
-        stats = super().sample_stats()
-        stats.update(
-            {
-                "accept_rate": self.acc_rate,
-            }
-        )
-        return stats
+        return {
+            "log_marginal_likelihood": self.log_marginal_likelihood if self.beta == 1 else np.nan,
+            "beta": self.beta,
+            "accept_rate": self.acc_rate,
+        }
 
     def sample_settings(self) -> SMCSettings:
-        stats = super().sample_settings()
-        stats.update(
-            {
-                "_n_tune": self.n_steps,  # Default property name used in `SamplerReport`
-                "correlation_threshold": self.correlation_threshold,
-            }
-        )
-        return stats
+        return {
+            "_n_draws": self.draws,
+            "threshold": self.threshold,
+            "_n_tune": self.n_steps,
+            "correlation_threshold": self.correlation_threshold,
+        }
 
 
 class Pearson:
@@ -470,6 +554,13 @@ class Pearson:
 class MH(SMC_KERNEL):
     """Metropolis-Hastings SMC_kernel."""
 
+    stats_dtypes_shapes: dict[str, tuple[type, list]] = {
+        "log_marginal_likelihood": (float, []),
+        "beta": (float, []),
+        "mean_accept_rate": (float, []),
+        "mean_proposal_scale": (float, []),
+    }
+
     def __init__(self, *args, correlation_threshold=0.01, **kwargs):
         """
         Create a Metropolis-Hastings SMC kernel.
@@ -556,24 +647,20 @@ class MH(SMC_KERNEL):
         self.chain_acc_rate = np.mean(ac_, axis=0)
 
     def sample_stats(self) -> SMCStats:
-        stats = super().sample_stats()
-        stats.update(
-            {
-                "mean_accept_rate": self.chain_acc_rate.mean(),
-                "mean_proposal_scale": self.proposal_scales.mean(),
-            }
-        )
-        return stats
+        return {
+            "log_marginal_likelihood": self.log_marginal_likelihood if self.beta == 1 else np.nan,
+            "beta": self.beta,
+            "mean_accept_rate": self.chain_acc_rate.mean(),
+            "mean_proposal_scale": self.proposal_scales.mean(),
+        }
 
     def sample_settings(self) -> SMCSettings:
-        stats = super().sample_settings()
-        stats.update(
-            {
-                "_n_tune": self.n_steps,  # Default property name used in `SamplerReport`
-                "correlation_threshold": self.correlation_threshold,
-            }
-        )
-        return stats
+        return {
+            "_n_draws": self.draws,
+            "threshold": self.threshold,
+            "_n_tune": self.n_steps,
+            "correlation_threshold": self.correlation_threshold,
+        }
 
 
 def systematic_resampling(weights, rng):
@@ -606,7 +693,7 @@ def systematic_resampling(weights, rng):
     return new_indices
 
 
-def _logp_forw(point, out_vars, in_vars, shared):
+def _logp_forw(point, out_vars, in_vars, shared, compile_kwargs=None):
     """Compile PyTensor function of the model and the input and output variables.
 
     Parameters
@@ -617,7 +704,12 @@ def _logp_forw(point, out_vars, in_vars, shared):
         Containing Distribution for the input variables
     shared : list
         Containing TensorVariable for depended shared data
+    compile_kwargs: dict, optional
+        Additional keyword arguments passed to pytensor.function
     """
+    if compile_kwargs is None:
+        compile_kwargs = {}
+
     # Replace integer inputs with rounded float inputs
     if any(var.dtype in discrete_types for var in in_vars):
         replace_int_input = {}
@@ -636,6 +728,6 @@ def _logp_forw(point, out_vars, in_vars, shared):
     out_list, inarray0 = join_nonshared_inputs(
         point=point, outputs=out_vars, inputs=in_vars, shared_inputs=shared
     )
-    f = compile([inarray0], out_list[0])
+    f = compile([inarray0], out_list[0], **compile_kwargs)
     f.trust_input = True
     return f
